module D5M_SET (
    input CLOCK_50,

	input    [11:0]D5M_D,
	input    D5M_FVAL,
	input 	D5M_LVAL,
	input 	D5M_PIXCLK,
	
	output	D5M_RESET_N,
	output	D5M_SCLK,
	inout 	D5M_SDATA,
	
	input 	D5M_STROBE,
	output	D5M_TRIGGER,
	output	D5M_XCLKIN,
	
	output	[11:0]	sCCD_R,
	output	[11:0]	sCCD_G,
	output	[11:0]	sCCD_B,
	output				CCD_DVAL, //FRAME & LINE VALID 
	output				CCD_LDVAL,//LINE VALID
	output				CCD_FDVAL,//FRAM VALID
	
   output   F_LED,
   output   reg OBJ_POINT,
	output   reg [41:0]oSUM ,
	input    RESET_SYS_N 
 	
);

assign CCD_LDVAL = mCCD_LDVAL ; 
assign CCD_FDVAL = mCCD_FDVAL ; 

reg	[11:0]	rCCD_DATA;
reg				rCCD_LVAL;
reg				rCCD_FVAL;
wire	[11:0]	mCCD_DATA;
wire				mCCD_DVAL;
wire				mCCD_DVAL_d;
wire	[15:0]	X_Cont;
wire	[15:0]	Y_Cont;
wire	[9:0]		X_ADDR;
wire	[31:0]	Frame_Cont;


//*** CLOCK _DIV **//
reg	[15:0]		rClk;
always@(posedge CLOCK_50)	rClk	<=	rClk+1;


//*** RESET1 **//
Reset_Delay			u2	(	
							.iCLK  (CLOCK_50),
							.iRST  (1),
							.oRST_0(),
							.oRST_1(D5M_RESET_N),
							.oRST_2()
						);
						
//*** RESET2 **//
reg [15:0]DELAY;
reg       START_TR ;
reg       D5M_RST ;
always @(posedge rClk[11])  begin
 if (!RESET_SYS_N ) DELAY <=0;
else
begin
 D5M_RST  <=  ( DELAY==16'h1) ?0:1 ;
 START_TR <=( ( DELAY>=16'hfff) && ( DELAY<(16'hfff + 1000))) ?1:0  ; 
 if (DELAY<16'hfff0) DELAY<=DELAY+1 ;
end
end

					
//*** D5M **//
assign	D5M_TRIGGER  =  1'b1 ;
assign   D5M_XCLKIN   = rClk[0]; //25M

//*** D5M **//
always@(posedge D5M_PIXCLK)
begin
	rCCD_DATA	<=	D5M_D;
	rCCD_LVAL	<=	D5M_LVAL;
	rCCD_FVAL	<=	D5M_FVAL;
end

//*** D5M **//
I2C_CCD_Config 		u8	(	//	Host Side
								 .iCLK			  (CLOCK_50),
								 .iRST_N         (D5M_RST),
								 .iZOOM_MODE_SW  (0),//SW[8]),
								 .iEXPOSURE_ADJ  (1),//KEY[1]),
								 .iEXPOSURE_DEC_p(0),//SW[0]),
								  //	I2C Side
								 .I2C_SCLK		  (D5M_SCLK),
								 .I2C_SDAT		  (D5M_SDATA)
							   );
						
//*** CCD PROCESSING **//						
CCD_Capture			u3	(	
							.oDATA      (mCCD_DATA),
							.oDVAL      (mCCD_DVAL),
							.oFDVAL     (mCCD_FDVAL ) , 							
							.oLDVAL     (mCCD_LDVAL ) , 
							
							.oX_Cont    (X_Cont),
							.oY_Cont    (Y_Cont),
							.oFrame_Cont( ),//Frame_Cont),
							.iDATA      (rCCD_DATA),
							.iFVAL      (rCCD_FVAL),
							.iLVAL      (rCCD_LVAL),
							.iSTART     (START_TR ),//!KEY[3]),//<------------------------START
							.iEND       (0),//!KEY[2]<------------------------END
							.iCLK       (D5M_PIXCLK),
							.iRST       (D5M_RST),
							.F_LED      (F_LED )    
							
						);
						
					
//*** CCD PROCESSING **//						
RAW2RGB				u4	(	
						   .iCLK   (D5M_PIXCLK),
							.iRST   (D5M_RST),
							.iDATA  (mCCD_DATA),
							.iDVAL  (mCCD_DVAL),
							.oRed   (sCCD_R),
							.oGreen (sCCD_G),
							.oBlue  (sCCD_B),
							.oDVAL  (sCCD_DVAL),
							.iX_Cont(X_Cont),
							.iY_Cont(Y_Cont)
						);
	


endmodule

