module FRAME_RAM_CTRL (

input D5M_PIXCLK , 
input D5M_FVAL,
input D5M_LVAL,
input CCD_DVAL,
input CCD_LDVAL ,
input CCD_FDVAL,
input [7:0]CCD_R ,
input [7:0]CCD_G ,
input [7:0]CCD_B ,


input [11:0] V_OFFSET,
input [11:0] H_OFFSET,

input [11:0] H_Cont ,
input [11:0] V_Cont ,

output  reg RD_VALID,
input   MTL_DCLK ,
input   MTL_VSD,
output [7:0] MTL_R,
output [7:0] MTL_G,
output [7:0] MTL_B
);
//-----
parameter  IMG_V  = 240;  //pixel,  320x240IMG's wide 
parameter  IMG_H  = 320;  //line,   320x240IMG's high
//--- FRAME BUFFER READ --- 
reg  [16:0]RD_ADDR ; 	
wire [23:0]RD_DATA ; 	
wire       RD_CLK ; 

//--- FRAME BUFFER WRITE --- 
reg  [16:0]WR_ADDR ; 	
wire [23:0]WR_DATA ; 	
wire       WR_CLK ; 

//-- FRAME BUFFER WRITE COUNTER --   
reg  [15:0]D5M_LCNT ; 
reg  [15:0]D5M_FCNT ; 
reg        rD5M_LVAL ; 

//----
assign {MTL_B[7:0],MTL_G[7:0],MTL_R[7:0] } = RD_DATA [23:0] ; 


assign     WR_DATA = {CCD_B[7:0] ,CCD_G[7:0] , CCD_R[7:0] };
assign     WR_CLK  = D5M_PIXCLK ; 


//--FRAME BUFFER WIRTE COUNTER --		
always @( posedge D5M_PIXCLK )  begin 
		rD5M_LVAL <= D5M_LVAL ; 
  if (!D5M_FVAL ) begin 
      WR_ADDR  <=0;
	   D5M_FCNT <=0; 
		D5M_LCNT <=0;
	  end 
  else  if ( rD5M_LVAL && !D5M_LVAL ) D5M_FCNT <= D5M_FCNT +1 ; 
  else  if ( D5M_LVAL )  begin 
        D5M_LCNT <=D5M_LCNT +1 ; 
        if ( 
		     ( D5M_FCNT >V_OFFSET && D5M_FCNT < V_OFFSET + IMG_V ) && 
		     ( D5M_LCNT > H_OFFSET && D5M_LCNT <H_OFFSET + IMG_H ) 
			  ) 
		         WR_ADDR <=WR_ADDR +1 ; 
    end 
	 else  D5M_LCNT <= 0 ; 
end		 

//---320x240 PIXEL FRAME BUFFER  --- 
FRAME_BUFFER fb(
	.data      (WR_DATA ),
	.wraddress (WR_ADDR),
	.wrclock   (WR_CLK),
	.wren   (1),
	.rdaddress (RD_ADDR),
	.rdclock   (MTL_DCLK),
	.q         (RD_DATA) 
	);	

	
	
//--FRAME BUFFER READ COUNTER --		
always @ ( posedge MTL_DCLK )begin
 if  (!MTL_VSD ) begin 
 RD_ADDR <=0;    
 RD_VALID <=0; 
 end
   else  
	begin 
	  if ( ( V_Cont >V_OFFSET && V_Cont <V_OFFSET+IMG_V   ) && (  H_Cont > H_OFFSET && H_Cont <H_OFFSET + IMG_H  ) ) 
     RD_ADDR <=RD_ADDR +1 ; 
	  
	  //mark 1 col vedio
	  if ( ( V_Cont > V_OFFSET && V_Cont <V_OFFSET+IMG_V   ) && (  H_Cont > H_OFFSET+1 && H_Cont <H_OFFSET + IMG_H  ) )  	 
       RD_VALID  <= 1; 
	      else 
       RD_VALID <=0; 
  end	 
end

endmodule 
